@@ -380,17 +380,21 @@ class Fsdb(object):
 
         # create the header line
         header_line = "#fsdb -F " + separator_token + " " + " ".join(columns) + "\n"
+        self._have_read_header = True
+
         return header_line
 
     # column accessor helpers
     def get_column_number(self, column_name):
         "Given a column_name, returns its integer index into an array of values."
         self.maybe_open_filehandle()
+        self.read_header()
         return self._column_names[column_name]
 
     def get_column_numbers(self, column_names):
         "Given a list of column_names, returns a list of integer index into an array of values."
         self.maybe_open_filehandle()
+        self.read_header()
         column_numbers = []
         for name in column_names:
             column_numbers.append(self.get_column_number(name))
@@ -399,6 +403,7 @@ class Fsdb(object):
     def get_column_name(self, column_number):
         "Given an integer column number, returns its column name."
         self.maybe_open_filehandle()
+        self.read_header()
         return self.column_nums[column_number]
 
     def set_iterator_function(self):
@@ -598,6 +603,7 @@ class Fsdb(object):
 
         if self._have_read_header:
             return [0, self._mapping]
+        self._have_read_header = True
 
         if not line:
             line = next(self.file_handle)
@@ -690,25 +696,27 @@ class Fsdb(object):
                                names=self.column_names,
                                usecols=usecols)
 
-    def foreach(self, fn, return_results=True):
+    def foreach(self, fn, return_results=True, args=[]):
         """Applies a function fn to each row, returning an 
         aggregate list of results if desired."""
         results = []
         if return_results:
             for row in self:
-                results.append(fn(row))
+                results.append(fn(row, *args))
         else:
             # do this separately for speed purposes
             for row in self:
-                fn(row)
+                fn(row, *args)
 
         return results
 
-    def filter(self, fn):
+    def filter(self, fn, args=[]):
         """Applies a function to the input stream rows, and saves the output
         results back to the output fsdb handle."""
         for row in self:
-            self.append(fn(row))
+            filtered = fn(row, *args)
+            if filtered:
+                self.append(filtered)
 
     #
     # writing functions
